import React, { Component, PropTypes } from 'react';
import {
  Image,
} from 'react-native';
import RNFetchBlob from 'react-native-fetch-blob';
import URI from 'urijs';
import _ from 'lodash';
import MD5 from 'crypto-js/md5';

const dirs = RNFetchBlob.fs.dirs;
const cacheRoot = `${dirs.DocumentDir}/react-native-yaimage`;

class YaImage extends Component {

  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      loaded: false,
    };

    this.cachePath = this.genPath(props.source, props.cachePathPattern);
    const uri = new URI(this.cachePath);
    if (uri.suffix()) {
      this.loadFromFilesystem = this.findWithSuffix;
      this.loadFromNetwork = this.fetchWithSuffix;
    } else {
      this.loadFromFilesystem = this.findWithoutSuffix;
      this.loadFromNetwork = this.fetchWithoutSuffix;
    }
  }

  componentDidMount() {
    this.loadFromFilesystem()
      .catch((r) => {
        return this.loadFromNetwork();
      })
      .then((path) => {
        this.setState({
          loading: false,
          loaded: true,
          source: { uri: `file://${path}` },
        });
      })
      .catch((r) => {
        this.setState({
          loading: false,
        });
      });
  }

  findWithSuffix() {
    const path = this.cachePath;
    return this.checkPath(path);
  }

  checkPath(path) {
    return new Promise((resolve, reject) => {
      RNFetchBlob.fs.exists(path)
        .then((exist) => {
          if (exist) {
            resolve(path);
          } else {
            reject('File not exists');
          }
        })
        .catch(r => reject(r));
    });
  }

  findWithoutSuffix() {
    const path = this.cachePath;
    const suffixesToTry = [
      'png',
      'jpg',
      'gif',
    ];

    return suffixesToTry.slice(1).reduce(
      (promise, suffix) => promise.catch(() => this.checkPath(`${path}.${suffix}`)),
      this.checkPath(`${path}.${suffixesToTry[0]}`));
  }

  genPath(source, cachePathPattern) {
    const uri = new URI(source.uri);
    const cachePath = _.template(cachePathPattern)({ uri, MD5 });
    const path = `${cacheRoot}/${cachePath}`;
    return path;
  }

  fetchWithSuffix() {
    return new Promise((resolve, reject) => {
      RNFetchBlob
        .config({
          // response data will be saved to this path if it has access right.
          path: this.cachePath,
          timeout: this.props.fetchTimeout,
        })
        .fetch('GET', this.props.source.uri, {
          //some headers ..
        })
        .then(res => resolve(res.path()))
        .catch((r) => {
          this.deleteCorruptedFile(this.cachePath);
          reject(r);
        });
    });
  }

  fetchWithoutSuffix() {
    return new Promise((resolve, reject) => {
      RNFetchBlob
        .config({
          // response data will be saved to this path if it has access right.
          path: this.cachePath,
          timeout: this.props.fetchTimeout,
        })
        .fetch('GET', this.props.source.uri, {
          //some headers ..
        })
        .then((res) => {
          // the path should be dirs.DocumentDir + 'path-to-file.anything'

          const contentType = res.respInfo.headers['Content-Type'];
          this.cachePath = `${this.cachePath}.${this.mimeToSuffix(contentType)}`;
          return this.mvFile(res.path(), this.cachePath);
        })
        .then(() => { resolve(this.cachePath); })
        .catch((r) => {
          this.deleteCorruptedFile(this.cachePath);
          reject(r);
        });
    });
  }

  mvFile(from, to) {
    return RNFetchBlob.fs.mv(from, to);
  }

  mimeToSuffix(contentType) {
    const [ majorType, minorType ] = contentType.toLowerCase().split('/');
    if (majorType === 'image') {
      const minorTypeToSuffix = {
        jpeg: 'jpg',
      };
      return minorTypeToSuffix[minorType] || minorType;
    }
    return minorType; // maybe for rare custom Image component
  }

  deleteCorruptedFile(path) {
    RNFetchBlob.fs.exists(path)
      .then((exist) => {
        if (exist) {
          RNFetchBlob.fs.unlink(path).then(() => {
            // console.log('Corrupted file deleted');
          });
        }
      });
  }

  render() {
    if (this.state.loading) {
      return this.props.renderLoadingIndicator();
    }

    if (this.state.loaded) {
      return this.props.renderLoadedImage(this.props, this.state.source);
    }

    return (this.props.renderLoadError && this.props.renderLoadError()) || this.props.renderLoadingIndicator();
  }
}

YaImage.propTypes = {
  ...Image.propTypes,
  cachePathPattern: PropTypes.string.isRequired,
  fetchTimeout: PropTypes.number,
  renderLoadingIndicator: PropTypes.func.isRequired,
  renderLoadError: PropTypes.func,
  renderLoadedImage: PropTypes.func.isRequired,
  imageComponent: PropTypes.func.isRequired,
};

YaImage.defaultProps = {
  cachePathPattern: '${uri.hostname()}/${MD5(uri.toString())}${uri.suffix() ? ("." + uri.suffix()) : ""}',
  renderLoadedImage: (props, source) => React.createElement(props.imageComponent, { ...props, source }),
  imageComponent: Image,
};

export default YaImage;
